/*
 * FlowAlarm.c
 *
 * Created: 9/28/2014 12:23:17 AM
 *  Author: Tom Blough tom@thebloughs.net
 *
 * **** Purpose:
 * This is a simple alarm program to monitor the output from a GEMS Rotorflow
 * sensor RFO155421.  This flow sensor has a Hall Effect sourcing pulse output
 * proportional to the flow.
 *
 * **** Introduction:
 * My TIG welder has a water-cooled torch.  Being water-cooled, it is lighter and 
 * easier to maneuver than an air-cooled torch because a much smaller cooling water
 * jacketed welding lead can be used to carry the welding current. The downside is
 * without cooling water flow, the smaller welding lead will overheat and melt the
 * cooling water hose.  This can be an expensive problem.
 *
 * The torch cooling water is supplied by a Miller WaterMate 1A chiller.  The
 * WaterMate comes with a simple sightglass to verify cooling water flow.  It is,
 * however, not in an easily visible location while welding.  Should the cooling
 * water hoses become blocked or pinched cutting off coolant flow while welding....
 *
 * This program and the simple circuit board that accompanies it, are designed to
 * monitor the flow of cooling water and provide an audible alert if water flow
 * is interrupted.
 *
 * With the TIG torch hoses removed from the WaterMate and a bypass hose
 * connecting the chiller output to input, the output from the RotorFlow sensor
 * was measured at 65Hz.  This can be considered to be the upper limit of flow
 * from the WaterMate.
 *
 * **** Theory of Operation:
 * It takes about 5 - 10 seconds after the pump on the WaterMate starts before
 * stable coolant flow is established.  To account for this the program will begin
 * with a 15 second delay where the piezo will chirp once every 2-seconds.  This
 * will verify both the program and the alert buzzer are operating.
 *
 * Once the startup delay is completed, the program will monitor the hall effect
 * output.  If the output drops below the min. frequency, the alarm will sound. 
 * To measure this, I'll use the Input Capture feature of the 16-bit timer to
 * measure the hall effect pulse period.  If the counter value is larger than
 * minimum flow or if it overflows the piezo output will be turned on.  Counter
 * values less than a minimum flow will turn the piezo off.
 *
 * This is a very simple program needing only a single input and a single output.
 * I/O will not be a problem, but to use ICP, we need a 16-bit timer/counter.  The
 * smallest pin-count AVR with a 16-bit T/C is the ATtiny24/44/84.  The program will
 * easily fit in the 2K flash of the ATtiny24, but with pricing of $1.45/$1.65/$1.91
 * respectively, for a hobbyist it does not make sense to keep anything other than the
 * 8K flash part on-hand.
 *
 */ 

// Target: ATtiny84
// intrcosc_8MHz, ckdiv8, spien
// fuses: ext/0xFF high/0xDF low/0x62

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "utility.h"

// program constants
#define F_CPU 1000000			// AVR clock frequency in Hz, used by util/delay.h
#define MIN_FLOW F_CPU/30		// timer counts for 30Hz minimum acceptable flow signal
#define STARTUP_DELAY_SEC 10	// seconds to wait after turn-on for flow to stabilize

// controller pin configuration
#define PIEZO(reg) BIT(A,2,reg)		// piezo bender for alarms
#define FREQ(reg) BIT(A,7,reg)		// PA7 is ICP on ATtiny84; GEM Rotorflow hall effect sensor
#define LED1(reg) BIT(A,3,reg)		// extra LED for whatever

// global variables
volatile static uint8_t G_overflow;	// counter overflow semaphore (accessed in ISRs)

ISR( TIM1_OVF_vect)	// overflow means frequency below 15Hz or no output from Hall sensor
{
	PIEZO(PORT) = ON;
	
	// since we've just overflowed, the next input rising edge will have an incorrect timer
	// count captured, so set a flag to discard it.
	G_overflow = TRUE;
}

ISR( TIM1_CAPT_vect)	// received a rising edge on the hall effect sensor input
{
	TCNT1 = 0;	// reset the counter to begin measuring the next input period
	
	// if we experienced an overflow, the period measurement is incorrect. So
	// ignore it, reset the overflow flag, and wait for the next pulse rising edge.
	if( ! G_overflow)
	{
		// good period measurement so see if our flow has dropped below the minimum
		if( ICR1 <= MIN_FLOW)
			PIEZO(PORT) = OFF;
		else
			PIEZO(PORT) = ON;
	}
	G_overflow = FALSE;
}


int main()
{
	OSCCAL = 0x8D;  // internal oscillator calibration for 8MHz/8
	
	// set all I/O to input and enable pullups - leave unused ports unconnected on PCB
	// this prevents noise and power consumption on unused pins.
	// we'll reconfigure any necessary pins later
	DDRA = 0x00;
	PORTA = 0xFF;
	DDRB = 0x00;
	PORTB = 0xFF;

	// setup inputs and outputs
	LED1(DDR) = OUTPUT;
	PIEZO(DDR) = OUTPUT;
	FREQ(DDR) = INPUT;
	
	LED1(PORT) = ON;	// for want of a better use, signal we've got power and are running
	
	// startup delay for flow to stabilize, beep for buzzer test
	for( uint8_t i = 0; i < STARTUP_DELAY_SEC/2; i++)  // run 2sec cycle nTimes
	{
		PIEZO(PORT) = ON;
		_delay_ms(50);
		PIEZO(PORT) = OFF;
		_delay_ms(1950);
	}
	
	// sound startup completion
	for( uint8_t i = 0; i < 3; i++)
	{
		PIEZO(PORT) = ON;
		_delay_ms(50);
		PIEZO(PORT) = OFF;
		_delay_ms(50);
	}
	
	// set up the 16-bit timer/counter
	// 65Hz pulse = 15.3846ms period, 20Hz pulse = 50.0ms period
	// With our 16-bit timer running with 1MHz clock and a 1X prescale, each timer
	// tick is 1ms.  Our 65Hz maximum frequency is within the scope of a 16-bit count.
	// Overflow will be at 65535ms (15.259Hz).
	TIMSK1 = (1 << TOIE1) | (1 << ICIE1);	// enable overflow and input capture interrupts
	TCCR1A = 0;	// normal timer mode
	TCCR1B = (1 << CS10) | (1 << ICES1) | (1 << ICNC1);	// no prescale, input capture on rising edge, noise canceling
	
	// chances are we will not enable interrupts right on a pulse leading edge
	// so set the overflow flag to cause the first pulse time to be discarded.
	G_overflow = TRUE;
	sei(); // Enable global interrupts	
	
	// main program loop
	while( TRUE)
	{
		// nothing to do - everything is handled in interrupts
	}
}

